package com.gct.tool.service;

import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.util.ResourceUtils;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
@Slf4j
public class BaseResourceService {

    private List<String> wellIds;

    private Map<String,String> urlMap;

    private Map<String,Map<String,String>> baseMap;

    public List<String> getWellIds() {
        return wellIds;
    }

    public Map<String,String> getUrlMap(){
        return urlMap;
    };

    public Map<String,Map<String,String>>getBaseMap(){return baseMap;};

    private void readWellId(){
        try {
            BufferedReader br = new BufferedReader(new InputStreamReader(getClass().getResourceAsStream("/well.txt")));
            String wellIdString = br.readLine();
            wellIds = Arrays.stream(wellIdString.split(",")).collect(Collectors.toList());
        } catch (Exception e) {
            e.printStackTrace();
            log.error("读取井号文件失败，程序退出");
            System.exit(0);
        }
    }

    private void readUrl(){
        try {
            urlMap = new HashMap<>();
            BufferedReader br = new BufferedReader(new InputStreamReader(getClass().getResourceAsStream("/url.txt")));
            String urlStr;
            while ((urlStr= br.readLine())!=null){
                String[] split = urlStr.split(",");
                urlMap.put(split[0],split[1]);
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error("读取URL文件失败，程序退出");
            System.exit(0);
        }
    }

    private void readBaseData(){
        try {
            baseMap = new HashMap<>();
            BufferedReader br = new BufferedReader(new InputStreamReader(getClass().getResourceAsStream("/base.txt")));
            String urlStr;
            while ((urlStr= br.readLine())!=null){
                String[] split = urlStr.split(",");
                Map<String, String> paramMap = new HashMap<>();
                paramMap.put("dym",split[1]);
                paramMap.put("bj",split[2]);
                paramMap.put("bs",split[3]);
                baseMap.put(split[0],paramMap);
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error("读取URL文件失败，程序退出");
            System.exit(0);
        }
    }

    public BaseResourceService(){
        readWellId();
        readBaseData();
        readUrl();
    }
}
