package com.gct.tool.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.gct.tool.entity.centralbase.CbCdWellSource;
import com.gct.tool.entity.centralbase.CbPcProWellboreStatusDaily;
import com.gct.tool.entity.centralbase.CbPcProWellboreVolDaily;
import com.gct.tool.entity.centralbase.CbTempWellMechRuntime;
import com.gct.tool.entity.sql.QueryBody;
import com.gct.tool.feign.DataBankClient;
import com.gct.tool.feign.DataBankTokenClient;
import com.gct.tool.feign.DiagnoseClient;
import com.gct.tool.feign.WorkDiagramClient;
import com.gct.tool.mapper.centralbase.CbCdWellSourceMapper;
import com.gct.tool.mapper.centralbase.CbPcProWellboreStatusDailyMapper;
import com.gct.tool.mapper.centralbase.CbPcProWellboreVolDailyMapper;
import com.gct.tool.mapper.centralbase.CbTempWellMechRuntimeMapper;
import lombok.extern.slf4j.Slf4j;
import org.apache.ibatis.logging.stdout.StdOutImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.stereotype.Service;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.time.Instant;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

@Service
@Slf4j
public class LoadBaseDataService implements ApplicationRunner {

    final String governDataServiceIdentity = "CDF03A45C9BF4D9B85155364FAAEE08C14D85F6D51774A59GMXF6MBQG4ZDOKZXL4YF6KZSGEZGEKZWMBQTMKZVL43WCYJVG4XWCMLDGUXT22DKO43TAZ3CMYXXEL3AMFWWWN3ROFXGEZZSONYDAMBTGEYTKLRVGEZTONZXFYXTAMBSF4YDAMQ";
    final String sharedDataServiceIdentity = "77ACF8C095A94D3CA3474B9561FEA62D352F67FAE44AF04CMA2TGLZPMJPTGKZUF4XTIKZSGRRDGK27MIZGAKZOF5RDGNZTGBQS4YRWF4XT2NDFGZ2DCNLIFYYXEY3XMBVG4NRSNR4DGZTEMRWTAMBTGEZC4MBQGUXTONZXFYXTAMBSF4YDAMQ";

    @Autowired
    BaseResourceService baseResourceService;

    @Autowired
    DataBankClient dataBankClient;
    @Autowired
    DataBankTokenClient tokenClient;

    @Autowired
    WorkDiagramClient workDiagramClient;

    @Autowired
    CbCdWellSourceMapper cbCdWellSourceMapper;

    @Autowired
    CbPcProWellboreStatusDailyMapper cbPcProWellboreStatusDailyMapper;

    @Autowired
    CbTempWellMechRuntimeMapper cbTempWellMechRuntimeMapper;

    @Autowired
    CbPcProWellboreVolDailyMapper cbPcProWellboreVolDailyMapper;

    @Autowired
    DiagnoseClient diagnoseClient;

    private static Map<String,Double> ymdMap=new HashMap<>();

    @Override
    public void run(ApplicationArguments args) throws Exception {
        BufferedReader br = new BufferedReader(new InputStreamReader(getClass().getResourceAsStream("/ymd.txt")));
        String line;
        while ((line=br.readLine())!=null){
            ymdMap.put(line.split("#")[0],Double.parseDouble(line.split("#")[1]));
        }
    }

    private void printMap(Object o, boolean isPrintNull, Set<String> filterSet) {
        ((Map) o).forEach((k, v) -> {
            if ((isPrintNull || Objects.nonNull(v)) && (Objects.isNull(filterSet) || filterSet.contains(k)))
                System.out.printf("%s:%s\n", k, v);
        });
    }

    /**
     * @return
     */
    public void loadCdWellSource() {
        List<String> wellIds = baseResourceService.getWellIds();
//        wellIds = wellIds.subList(0, 1);
        Set<String> filterSet = new HashSet<>();
        filterSet.addAll(Arrays.asList("WELL_ID", "WELL_COMMON_NAME", "WELL_LEGAL_NAME"));
        wellIds.forEach(wellId -> {
            QueryBody queryBody = new QueryBody();
            queryBody.select("TL").eq("WELL_LEGAL_NAME", wellId);
            JSONObject object = dataBankClient.getCdWellData(queryBody.toJsonMap(), 0, 10, governDataServiceIdentity);
            JSONArray array = object.getJSONObject("resultData").getJSONArray("content");

            array.forEach(o -> {
                printMap(o, false, null);
            });
        });
    }

    public void loadVolDailyData(LocalDateTime date) {
        List<CbCdWellSource> cbCdWellSources = cbCdWellSourceMapper.selectList(new QueryWrapper<>());
        cbCdWellSources.forEach(cbCdWellSource -> {
            QueryBody queryBody = new QueryBody();
            String dateStr = date.minusDays(1).format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
            String dateStrEd = date.plusDays(1).format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
            queryBody.select("TL").eq("WELL_ID", cbCdWellSource.getWellId()).gt("PROD_DATE", dateStr).lt("PROD_DATE",dateStrEd);
            JSONObject object = dataBankClient.getVolDailyData(queryBody.toJsonMap(), 0, 500, governDataServiceIdentity);
            JSONArray array = object.getJSONObject("resultData").getJSONArray("content");
            for (int i = 0; i < array.size(); i++) {
                JSONObject volDailyJson = array.getJSONObject(i);
                CbPcProWellboreVolDaily volDaily = volDailyJson.toJavaObject(CbPcProWellboreVolDaily.class);
                volDaily.setWaterCut(volDailyJson.getDouble("WATER_CUT_RATIO"));
                volDaily.setSurfaceCrudeOilDensity(ymdMap.getOrDefault(cbCdWellSource.getWellLegalName(), 0.84));
                volDaily.setSurfaceCrudeWaterDensity(1.0);
                if (Objects.isNull(volDaily.getGasOilRatio()))volDaily.setGasOilRatio(0d);
                if (Objects.isNull(volDaily.getWaterGasRatio()))volDaily.setWaterGasRatio(0d);
                QueryWrapper<CbPcProWellboreVolDaily> wrapper = new QueryWrapper<CbPcProWellboreVolDaily>()
                        .eq("well_id", volDaily.getWellId())
                        .eq("prod_date", volDaily.getProdDate());
                if (Objects.isNull(cbPcProWellboreVolDailyMapper.selectOne(wrapper))) {
                    cbPcProWellboreVolDailyMapper.insert(volDaily);
                } else {
                    cbPcProWellboreVolDailyMapper.update(volDaily, wrapper);
                }
            }

        });
    }

    public void loadStatusDailyData(LocalDateTime date) {
        List<CbCdWellSource> cbCdWellSources = cbCdWellSourceMapper.selectList(new QueryWrapper<>());
//        cbCdWellSources = cbCdWellSources.subList(0, 1);
        cbCdWellSources.forEach(cbCdWellSource -> {
            CbPcProWellboreStatusDaily statusDaily = new CbPcProWellboreStatusDaily();
            //get data from subjectdataservice/X_OP_PRO_WELL_STATUS_DAILY
            QueryBody queryBody = new QueryBody();
            String dateStr = date.format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
            String dateStrEd = date.plusDays(1).format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
            queryBody.select("TL").eq("WELL_ID", cbCdWellSource.getWellId()).gt("PROD_DATE", dateStr).lt("PROD_DATE",dateStrEd);
            JSONObject object = dataBankClient.getStatusDailyData1(queryBody.toJsonMap(), 0, 5, sharedDataServiceIdentity);
            JSONArray array = object.getJSONObject("resultData").getJSONArray("content");
            LocalDateTime nearlyTime = LocalDateTime.now().minusDays(4);
            for (int i = 0; i < array.size(); i++) {
                JSONObject obj = array.getJSONObject(i);
                LocalDateTime prodDate = LocalDateTime.parse(obj.getString("PROD_DATE"),DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
                if (prodDate.isAfter(nearlyTime)){
                    nearlyTime = prodDate;
                    statusDaily.setWellId(cbCdWellSource.getWellId());
                    statusDaily.setProdDate(prodDate);
                    statusDaily.setTubingPres(Double.parseDouble(obj.getString("TUBING_PRES")));
                    statusDaily.setCasingPres(Double.parseDouble(obj.getString("CASING_PRES")));
                    Map<String, Map<String, String>> baseMap = baseResourceService.getBaseMap();
                    Map<String, String> map = baseMap.get(cbCdWellSource.getWellLegalName());
                    if (Objects.nonNull(map)){
                        statusDaily.setStartPumpLiqLevel(Double.parseDouble(map.get("dym")));
                        statusDaily.setPumpDepth(Double.parseDouble(map.get("bs")));
                        statusDaily.setOilNozzle(Double.parseDouble(map.get("bj")));
                        statusDaily.setSubmergenceDepth(statusDaily.getPumpDepth()-statusDaily.getStartPumpLiqLevel());
                    }
                }
            }
            if (Objects.nonNull(statusDaily.getWellId())){
                QueryWrapper<CbPcProWellboreStatusDaily> wrapper = new QueryWrapper<CbPcProWellboreStatusDaily>()
                        .eq("well_id", statusDaily.getWellId()).eq("prod_date", statusDaily.getProdDate());
                if (Objects.isNull(cbPcProWellboreStatusDailyMapper.selectOne(wrapper))){
                    cbPcProWellboreStatusDailyMapper.insert(statusDaily);
                }else {
                    cbPcProWellboreStatusDailyMapper.update(statusDaily,wrapper);
                }
            }

        });
    }

    private JSONArray executeRemoteSql(String sqlString){
        String res = workDiagramClient.getWorkDiagramData(sqlString);
        res=res.replace("\\","");
//        res=res.substring(1,res.length()-1);
//        System.out.println(res);
        JSONObject selectMaxTimeJson = JSONObject.parseObject(res);
        return selectMaxTimeJson.getJSONArray("dataList");
    }

    public void loadWorkDiagramData(){
        List<CbCdWellSource> cbCdWellSources = cbCdWellSourceMapper.selectList(new QueryWrapper<>());
        System.out.println("load work diagram task execute begin --> well size + " + cbCdWellSources.size());
        cbCdWellSources.forEach(cbCdWellSource -> {
            System.out.println("----->load well gt"+cbCdWellSource.getWellLegalName());
            String dateStrSt = LocalDateTime.now().minusDays(1).format(DateTimeFormatter.ofPattern("yyyy-MM-dd"));
            String dateStrEd = LocalDateTime.now().plusDays(1).format(DateTimeFormatter.ofPattern("yyyy-MM-dd"));
           try{
               String selectNearlyWorkDiagramSqlStr = String.format(
                       "select * from PC_FD_PUMPJACK_DYNA_DIA_T@A11gxk where DYNA_CREATE_TIME >=  to_date('%s','yyyy-MM-dd') and DYNA_CREATE_TIME <=to_date('%s','yyyy-MM-dd')"+
                               "and WELL_COMMON_NAME = '%s'",
                       dateStrSt,dateStrEd,cbCdWellSource.getWellCommonName().contains("英买")?cbCdWellSource.getWellCommonName():cbCdWellSource.getWellLegalName());
               JSONArray array = executeRemoteSql(selectNearlyWorkDiagramSqlStr);
//            System.out.println(array);
               Integer gtCount = 0;
               for (int i = 0; i < array.size(); i++) {
                   JSONObject nearlyWorkDiagram = array.getJSONObject(i);
                   String displacement = nearlyWorkDiagram.getString("DISPLACEMENT");
                   String load = nearlyWorkDiagram.getString("DISP_LOAD");
                   StringBuilder sgt = new StringBuilder();
                   List<Double> dispacementList = Arrays.stream(displacement.split(";")).map(Double::parseDouble).collect(Collectors.toList());
                   List<Double> loadList = Arrays.stream(load.split(";")).map(Double::parseDouble).collect(Collectors.toList());
                   for (int j=0;j<Math.min(dispacementList.size(),loadList.size());j++){
                       sgt.append((int)(dispacementList.get(j)*100)).append(",").append((int)(loadList.get(j)*100)).append(",");
                   }
                   sgt.delete(sgt.length()-1,sgt.length());
                   Double stroke = Double.parseDouble(nearlyWorkDiagram.getString("STROKE"));
                   Double frequency = Double.parseDouble(nearlyWorkDiagram.getString("FREQUENCY"));
                   CbTempWellMechRuntime mechRuntime = new CbTempWellMechRuntime();
                   mechRuntime.setWellId(cbCdWellSource.getWellId());

                   mechRuntime.setProdDate(LocalDateTime.ofInstant(Instant.ofEpochSecond(nearlyWorkDiagram.getLong("DYNA_CREATE_TIME")/1000),TimeZone.getDefault().toZoneId()));
                   mechRuntime.setSgt(sgt.toString());
                   mechRuntime.setStrokeLength(stroke);
                   mechRuntime.setStrokeFrequency(frequency);

                   QueryWrapper<CbTempWellMechRuntime> wrapper = new QueryWrapper<CbTempWellMechRuntime>()
                           .eq("well_id", mechRuntime.getWellId())
                           .eq("prod_date", mechRuntime.getProdDate());
                   List<CbTempWellMechRuntime> list = cbTempWellMechRuntimeMapper.selectList(wrapper);
                   DateTimeFormatter pattern = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
                   if (Objects.isNull(list)||list.isEmpty()){
                       gtCount++;
                       cbTempWellMechRuntimeMapper.insert(mechRuntime);
                       diagnoseClient.runDgnsByDateJh(1,mechRuntime.getProdDate().minusMinutes(1).format(pattern),mechRuntime.getProdDate().plusMinutes(1).format(pattern),mechRuntime.getWellId());
                   }else {
                       if (!mechRuntime.toString().equals(list.get(0).toString())){
                           diagnoseClient.runDgnsByDateJh(1,mechRuntime.getProdDate().minusMinutes(1).format(pattern),mechRuntime.getProdDate().plusMinutes(1).format(pattern),mechRuntime.getWellId());
                           cbTempWellMechRuntimeMapper.update(mechRuntime,wrapper);
                       }
                   }
               }
               System.out.printf("<----well name: %s time : %s, gt count : %s \n",cbCdWellSource.getWellCommonName(),LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")),gtCount);

           }catch (Exception e){
               log.error("load work diagram error {},",e.getMessage());
           }
        });
    }


}

